package org.eclipse.wtp.releng.tools;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;


public class CreateFeatureUpdateFile {

	final private static String FEATURES_PROPERTY = "featuresLocation";
	final private static String EOL = System.getProperty("line.separator");

	private String rawUpdateSite;
	private int returnCode;
	private String featuresDirectoryLocation; 
	private FullJarNameParser fullJarNameParser = new FullJarNameParser();

	private String[] featuresOfInterest = new String[]{"org.eclipse.wst", "org.eclipse.jst", "org.eclipse.jpt.feature", "org.eclipse.jpt.eclipselink.feature", "org.eclipse.wst.xml_ui.feature", "org.eclipse.wst.jsdt.feature", "org.eclipse.wst.common_ui.feature", "org.eclipse.jst.webpageeditor.feature", "org.eclipse.jst.jsf.apache.trinidad.tagsupport.feature", "org.eclipse.jst.ws.axis2tools.feature", "org.eclipse.wst.sdk", "org.eclipse.jst.sdk", "org.eclipse.jpt_sdk.feature", "org.eclipse.jpt.eclipselink_sdk.feature", "org.eclipse.wst.xml_sdk.feature", "org.eclipse.wst.jsdt_sdk.feature", "org.eclipse.wst.common_sdk.feature", "org.eclipse.jst.webpageeditor_sdk.feature", "org.eclipse.jst.jsf.apache.trinidad.tagsupport_sdk.feature", "org.eclipse.jst.ws.axis2tools_sdk.feature",};

	private String[] shortListOfInterest = new String[]{"org.eclipse.wst", "org.eclipse.jst", "org.eclipse.jpt.feature", "org.eclipse.jpt.eclipselink.feature", "org.eclipse.wst.xml_ui.feature", "org.eclipse.wst.jsdt.feature", "org.eclipse.wst.common_ui.feature", "org.eclipse.jst.webpageeditor.feature", "org.eclipse.jst.jsf.apache.trinidad.tagsupport.feature", "org.eclipse.jst.ws.axis2tools.feature",};

	public static void main(String[] args) {

		if (args.length != 1) {
			System.out.println("    Error: Program needs rawUpdateSite");
		}
		else {
			try {
				new CreateFeatureUpdateFile().createFeatureFiles(args);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void createFeatureFiles(String[] args) throws IOException {
		rawUpdateSite = args[0];
		featuresDirectoryLocation = rawUpdateSite + "/features";

		System.out.println("featuresDirectoryLocation: " + featuresDirectoryLocation);
		File featuresDirectory = new File(featuresDirectoryLocation);
		if (!featuresDirectory.exists()) {
			System.out.println("ERROR: no directory at location: " + featuresDirectoryLocation);
		}
		else {
			String[] featuresDirectories = featuresDirectory.list();
			if (featuresDirectories == null || featuresDirectories.length <= 0) {
				System.out.println("ERROR: no featues found at location: " + featuresDirectoryLocation);
				returnCode = 9;
			}
			else {
				String filename = "features-ganymede-wtp.xml";
				System.out.println("Creating full List for main WTP site");
				System.out.println("   filename: " + filename);
				doList(featuresDirectories, featuresOfInterest, filename);
				System.out.println();


				// System.out.println("Short List for Europa site");
				// System.out.println();
				// doList(featuresDirectories, shortListOfInterest);

				String scfilename="wtp.sc";
				System.out.println("Creating short List for Ganymede site");
				System.out.println("   filename: " + scfilename);
				doList2(featuresDirectories, shortListOfInterest, scfilename);
			}
		}
		System.exit(returnCode);
	}

	private void doList(String[] featureDirectories, String[] featureOfFocus, String filename) throws IOException {
		File outFile = new File(filename);
		Writer output = new FileWriter(outFile);
		try {
		output.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + EOL + "<project" + EOL + "    name=\"update\"" + EOL + "    default=\"update\">" + EOL + "    <property" + EOL + "        environment=\"env\" />" + EOL + "    <target" + EOL + "        name=\"update\">" + EOL + "        <echo" + EOL + "            message=\"-------- Project: WTP Responsible: david_williams\" />" + EOL + "		<!-- intentionally no site.xml in this particular site -->" + EOL + "        <!--" + EOL + "            don't forget, for now have to first manually copy xsl" + EOL + "            bundles to 3.0 site" + EOL + "        -->" + EOL + "        <property" + EOL + "            name=\"from.update.site\"" + EOL + 
					"            value=\"file://" + rawUpdateSite + "\" />" + EOL + "        <echo" + EOL + "            message=\"   pulling update jars from ${from.update.site}\" />" + EOL);
		for (int i = 0; i < featureDirectories.length; i++) {
			String directoryName = featureDirectories[i];
			fullJarNameParser.parse(directoryName);
			String projectName = fullJarNameParser.getProjectString();
			String versionString = fullJarNameParser.getVersionString();
			if (contains(projectName, featureOfFocus)) {
				output.write("        <ant antfile=\"updateMirrorProject.xml\">" + EOL + "            <property" + EOL + "                name=\"featureId\"" + EOL + "                value=\"" + projectName + "\" />" + EOL + "            <property" + EOL + "                name=\"version\"" + EOL + "                value=\"" + versionString + "\" />" + EOL + "        </ant>" + EOL);

			}
		}
		output.write("    </target>" + EOL + "</project>");
		}
		finally {
			if (output != null) {
				output.close();
			}
		}
	}

	private boolean contains(String needle, String[] haystack) {
		boolean result = false;

		for (int i = 0; i < haystack.length; i++) {
			if (needle.equals(haystack[i])) {
				result = true;
				break;
			}
		}
		return result;
	}

	private void doList2(String[] featureDirectories, String[] featureOfFocus, String filename) throws IOException {
		
		String scfilehead = "<?xml version='1.0' encoding=\"UTF-8\"?>" + EOL + 
		"<sc:siteContribution" + EOL + 
		"    xmlns=\"http://www.eclipse.org/buckminster/CSpec-1.0\"" + EOL + 
		"    xmlns:sc=\"http://www.eclipse.org/buckminster/SiteContribution-1.0\"" + EOL + 
		"    updateSite=\"${downloads}/webtools/tempTestUpdates/site.xml\"" + EOL + 
		"    projectId=\"wtp\">" + EOL + 
		"    <sc:member" + EOL + 
		"        name=\"David Williams\"" + EOL + 
		"        email=\"david_williams@us.ibm.com\" />" + EOL + 
		"    <sc:cspec" + EOL + 
		"        name=\"org.eclipse.wtp-sc\">" + EOL + 
		"        <dependencies>" + EOL;
		
		String scfiletail="        </dependencies>" + EOL + 
				"        <groups>" + EOL + 
				"            <public" + EOL + 
				"                name=\"Web and Java EE Development\">" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.wst.xml_ui.feature\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.wst.jsdt.feature\" />    " + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.wst\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jst\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jpt.feature\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jpt.eclipselink.feature\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jst.webpageeditor.feature\" />   " + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jst.jsf.apache.trinidad.tagsupport.feature\" />   " + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jst.ws.axis2tools.feature\" />   " + EOL + 
				"            </public>" + EOL + 
				"            <public" + EOL + 
				"                name=\"Enabling Features\">" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.wst.common_ui.feature\" />" + EOL + 
				"            </public>" + EOL + 
				"            <public" + EOL + 
				"                name=\"Java Development\">" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jpt.feature\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jpt.eclipselink.feature\" />" + EOL + 
				"                <attribute" + EOL + 
				"                    component=\"org.eclipse.jst\" />" + EOL + 
				"            </public>" + EOL + 
				"        </groups>" + EOL + 
				"    </sc:cspec>" + EOL + 
				"</sc:siteContribution>" ;

		File outFile = new File(filename);
		Writer output = new FileWriter(outFile);
		try {
		output.write(scfilehead);
		
		for (int i = 0; i < featureDirectories.length; i++) {
			String directoryName = featureDirectories[i];
			fullJarNameParser.parse(directoryName);
			String projectName = fullJarNameParser.getProjectString();
			String versionString = fullJarNameParser.getVersionString();
			if (contains(projectName, featureOfFocus)) {
				output.write("\t\t\t" + "<dependency name=\"" + projectName + "\" versionDesignator=\"[" + versionString + "]\" />" + EOL);
			}
		}
		output.write(scfiletail);
		}
		finally {
			if (output != null) {
				output.close();
			}
		}
		
	}
}
